;; Generated package description from typst-ts-mode.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "typst-ts-mode" "0.12.2.0.20260415.103532" "Tree Sitter support for Typst" '((emacs "29.1")) :commit "278562d702de429f5c4369c007913ca0ef1584f3" :authors '(("Ziqi Yang" . "mr.meowking@anche.no")) :maintainer '(("Ziqi Yang" . "mr.meowking@anche.no") ("Huan Nguyen" . "nguyenthieuhuan@gmail.com")) :keywords '("typst" "languages" "tree-sitter") :url "https://codeberg.org/meow_king/typst-ts-mode")
